"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
exports.__esModule = true;
exports.introspectMachine = exports.getMatchesStates = void 0;
require("colors");
var utils_1 = require("xstate/lib/utils");
var traversalUtils_1 = require("./traversalUtils");
exports.getMatchesStates = function (machine) {
    var allStateNodes = machine.stateIds.map(function (id) {
        return machine.getStateNodeById(id);
    });
    var states = allStateNodes.reduce(function (arr, node) {
        return __spreadArrays(arr, utils_1.toStatePaths(utils_1.pathToStateValue(node.path)).map(function (path) {
            return path.join('.');
        }));
    }, []);
    return states;
};
var makeSubStateFromNode = function (node, rootNode, nodeMaps) {
    var nodeFromMap = nodeMaps[node.id];
    var stateNode = rootNode.getStateNodeById(node.id);
    var targets = traversalUtils_1.getTransitionsFromNode(stateNode);
    return {
        sources: Array.from(nodeFromMap.sources)
            .filter(Boolean)
            .map(function (event) { return "'" + event + "'"; })
            .join(' | ') || 'never',
        targets: Array.from(targets)
            .filter(Boolean)
            .map(function (event) { return "'" + event + "'"; })
            .join(' | ') || 'never',
        states: Array.from(nodeFromMap.children).reduce(function (obj, child) {
            var _a;
            var childNode = rootNode.getStateNodeById(child);
            return __assign(__assign({}, obj), (_a = {}, _a[childNode.key] = makeSubStateFromNode(childNode, rootNode, nodeMaps), _a));
        }, {})
    };
};
var xstateRegex = /^xstate\./;
exports.introspectMachine = function (machine) {
    var actionMaps = {};
    var condMaps = {};
    var servicesMaps = {};
    var nodeMaps = {};
    var activities = [];
    var allStateNodes = machine.stateIds.map(function (id) {
        return machine.getStateNodeById(id);
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        nodeMaps[node.id] = {
            sources: new Set(),
            children: new Set()
        };
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        var _a, _b, _c, _d;
        (_a = Object.values(node.states)) === null || _a === void 0 ? void 0 : _a.forEach(function (childNode) {
            nodeMaps[node.id].children.add(childNode.id);
        });
        // TODO - make activities pick up the events
        // that led to them
        (_b = node.activities) === null || _b === void 0 ? void 0 : _b.forEach(function (activity) {
            if (/\./.test(activity.type))
                return;
            if (activity.type && activity.type !== 'xstate.invoke') {
                activities.push(activity.type);
            }
        });
        (_c = node.invoke) === null || _c === void 0 ? void 0 : _c.forEach(function (service) {
            if (typeof service.src !== 'string' || /\./.test(service.src))
                return;
            if (!servicesMaps[service.src]) {
                servicesMaps[service.src] = new Set();
            }
        });
        (_d = node.transitions) === null || _d === void 0 ? void 0 : _d.forEach(function (transition) {
            var _a, _b, _c;
            (_a = transition.target) === null || _a === void 0 ? void 0 : _a.forEach(function (targetNode) {
                nodeMaps[targetNode.id].sources.add(transition.eventType);
            });
            if (transition.cond && transition.cond.name) {
                if (transition.cond.name !== 'cond') {
                    if (!condMaps[transition.cond.name]) {
                        condMaps[transition.cond.name] = new Set();
                    }
                    condMaps[transition.cond.name].add(transition.eventType);
                }
            }
            (_b = transition.target) === null || _b === void 0 ? void 0 : _b.forEach(function (targetNode) {
                var _a;
                /** Pick up invokes */
                (_a = targetNode.invoke) === null || _a === void 0 ? void 0 : _a.forEach(function (service) {
                    if (typeof service.src !== 'string' || /\./.test(service.src))
                        return;
                    if (!servicesMaps[service.src]) {
                        servicesMaps[service.src] = new Set();
                    }
                    servicesMaps[service.src].add(transition.eventType);
                });
            });
            if (transition.actions) {
                (_c = transition.actions) === null || _c === void 0 ? void 0 : _c.forEach(function (action) {
                    if (!xstateRegex.test(action.type)) {
                        if (!actionMaps[action.type]) {
                            actionMaps[action.type] = new Set();
                        }
                        actionMaps[action.type].add(transition.eventType);
                    }
                    return {
                        name: action.type,
                        event: transition.eventType
                    };
                });
            }
        });
    });
    allStateNodes === null || allStateNodes === void 0 ? void 0 : allStateNodes.forEach(function (node) {
        var _a;
        var allActions = [];
        allActions.push.apply(allActions, node.onExit);
        allActions.push.apply(allActions, node.onEntry);
        allActions === null || allActions === void 0 ? void 0 : allActions.forEach(function (action) {
            if (xstateRegex.test(action.type) || action.exec)
                return;
            if (!actionMaps[action.type]) {
                actionMaps[action.type] = new Set();
            }
        });
        (_a = node.onEntry) === null || _a === void 0 ? void 0 : _a.forEach(function (action) {
            var sources = nodeMaps[node.id].sources;
            sources === null || sources === void 0 ? void 0 : sources.forEach(function (source) {
                if (!actionMaps[action.type]) {
                    /* istanbul ignore next */
                    actionMaps[action.type] = new Set();
                }
                actionMaps[action.type].add(source);
            });
        });
    });
    var condLines = Object.entries(condMaps)
        .filter(function (_a) {
        var name = _a[0];
        return !/\./.test(name);
    })
        .map(function (_a) {
        var name = _a[0], eventSet = _a[1];
        return {
            name: name,
            events: Array.from(eventSet).filter(Boolean)
        };
    });
    var actionLines = Object.entries(actionMaps)
        .filter(function (_a) {
        var name = _a[0];
        return !/\./.test(name);
    })
        .map(function (_a) {
        var name = _a[0], eventSet = _a[1];
        return {
            name: name,
            events: Array.from(eventSet).filter(Boolean)
        };
    });
    var serviceLines = Object.entries(servicesMaps)
        .filter(function (_a) {
        var name = _a[0];
        return !/\./.test(name);
    })
        .map(function (_a) {
        var name = _a[0], serviceSet = _a[1];
        return {
            name: name,
            events: Array.from(serviceSet).filter(Boolean)
        };
    });
    var subState = makeSubStateFromNode(machine, machine, nodeMaps);
    return {
        stateMatches: exports.getMatchesStates(machine),
        subState: subState,
        condLines: condLines,
        actionLines: actionLines,
        services: serviceLines,
        activities: Array.from(activities)
    };
};
