"use strict";
exports.__esModule = true;
var compiled_1 = require("@xstate/compiled");
var react_1 = require("@xstate/compiled/react");
var lightMachine = compiled_1.Machine({
    initial: 'green',
    context: { elapsed: 0 },
    id: 'superMachine',
    on: {
        POWER_OUTAGE: {
            target: '.red'
        }
    },
    states: {
        green: {
            on: {
                TIMER: 'yellow',
                POWER_OUTAGE: 'red'
            }
        },
        yellow: {
            on: {
                TIMER: 'red',
                POWER_OUTAGE: 'red',
                PED_COUNTDOWN: 'red'
            }
        },
        red: {
            on: {
                TIMER: 'green',
                POWER_OUTAGE: '.stop'
            },
            initial: 'walk',
            states: {
                walk: {
                    on: {
                        PED_COUNTDOWN: 'wait'
                    }
                },
                wait: {
                    on: {
                        PED_COUNTDOWN: {
                            cond: 'isSuperCool',
                            target: 'wait'
                        }
                    }
                },
                stop: {
                    always: {
                        target: 'walk'
                    }
                }
            }
        }
    }
});
var useTrafficLightMachine = function () {
    // We use useCompiledMachine instead of
    // useMachine to avoid function overload problems
    var _a = react_1.useMachine(lightMachine, {
        guards: {
            isSuperCool: function (context, event) {
                // Note that the event here is typed exactly
                // to where the guard is used.
                return event.duration === 0 && context.elapsed > 0;
            }
        }
    }), state = _a[0], send = _a[1];
    return [state, send];
};
var interpretTrafficLightMachine = function () {
    // We use interpretCompiled instead of
    // interpret to avoid function overload problems
    var interpreter = compiled_1.interpret(lightMachine, {
        guards: {
            isSuperCool: function (context, event) {
                // Note that the event here is typed exactly
                // to where the guard is used.
                return event.duration === 0 && context.elapsed > 0;
            }
        }
    });
    return interpreter;
};
